//=============================================================================
//  MuseScore Plugin for Guitar Colour System
//
//  Copyright (C) 2015 Burkhard Mikolai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 3.0

MuseScore {
      version:  "1.0"
      description: "This plugin colors notes for use with Guitar Colour System (www.bunte-noten.de)"
      menuPath: "Plugins.GuitarColour.7th Position"

      property variant colors : [
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#000000", "#000000", "#000000",
               "#804000", "#804000", "#804000",
               "#804000", "#804000", "#0035ff",
               "#0035ff", "#0035ff", "#0035ff",
               "#0035ff", "#ff0000", "#ff0000",
               "#ff0000", "#ff0000", "#ff0000",
               "#00A700", "#00A700", "#00A700",
               "#00A700", "#ffcc00", "#ffcc00",
               "#ffcc00", "#ffcc00", "#ffcc00",
               "#ffcc00", "#ffcc00", "#ffcc00",
               "#ffcc00", "#ffcc00", "#ffcc00",
               "#ffcc00", "#ffcc00", "#ffcc00",
               "#ffcc00", "#ffcc00", "#ffcc00",
               "#ffcc00", "#ffcc00", "#ffcc00"
               ]

      // Apply the given function to all notes in selection
      // or, if nothing is selected, in the entire score

      function applyToNotesInSelection(func) {
            var cursor = curScore.newCursor();
            cursor.rewind(1);
            var startStaff;
            var endStaff;
            var endTick;
            var fullScore = false;
            if (!cursor.segment) { // no selection
                  fullScore = true;
                  startStaff = 0; // start with 1st staff
                  endStaff = curScore.nstaves - 1; // and end with last
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2);
                  if (cursor.tick == 0) {
                        // this happens when the selection includes
                        // the last measure of the score.
                        // rewind(2) goes behind the last segment (where
                        // there's none) and sets tick=0
                        endTick = curScore.lastSegment.tick + 1;
                  } else {
                        endTick = cursor.tick;
                  }
                  endStaff = cursor.staffIdx;
            }
            console.log(startStaff + " - " + endStaff + " - " + endTick)
            for (var staff = startStaff; staff <= endStaff; staff++) {
                  for (var voice = 0; voice < 4; voice++) {
                        cursor.rewind(1); // sets voice to 0
                        cursor.voice = voice; //voice has to be set after goTo
                        cursor.staffIdx = staff;

                        if (fullScore)
                              cursor.rewind(0) // if no selection, beginning of score

                        while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                              if (cursor.element && cursor.element.type == Element.CHORD) {

                                    var graceChords = cursor.element.graceNotes;
                                    for (var i = 0; i < graceChords.length; i++) {
                                          // iterate through all grace chords
                                          var notes = graceChords[i].notes;
                                          func(note);
                                    }
                                    var notes = cursor.element.notes;
                                    for (var i = 0; i < notes.length; i++) {
                                          var note = notes[i];
                                          func(note);
                                    }

                              }
                              cursor.next();
                        }
                  }
            }
      }

      function colorNote(note) {
         // color notes
         if (note.pitch < colors.length) {
            note.color = colors[note.pitch];
         }
      }

      onRun: {
            if (typeof curScore === 'undefined')
                  Qt.quit();

            applyToNotesInSelection(colorNote)

            Qt.quit();
         }
}
